"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataFrame = void 0;
const jsdom_1 = require("jsdom");
class DataFrame {
    constructor(html, options) {
        this.html = html;
        this.dom = new jsdom_1.JSDOM(html);
        this.document = this.dom.window.document;
        this.options = options;
    }
    validateHtml() {
        if (!this.html || this.html === '') {
            throw new Error('HTML cannot be empty');
        }
    }
    /**
     * Validates the provided headers against the number of columns in the table.
     * Throws an error if the lengths do not match.
     *
     * @param headers - The headers provided by the user.
     * @param document - The HTML document containing the table.
     */
    validateHeaders(headers) {
        const columnCount = this.document.querySelectorAll('table thead th, , table thead td').length;
        if (headers.length !== columnCount) {
            throw new Error(`The number of provided headers (${headers.length}) does not match the number of columns in the table (${columnCount}).`);
        }
    }
    build() {
        var _a, _b;
        let headers;
        if (((_a = this.options) === null || _a === void 0 ? void 0 : _a.header) && this.options.header.length > 0) {
            headers = this.options.header;
            this.validateHeaders(headers);
        }
        else {
            headers = this.generateHeaders(); // Generate headers if not provided
        }
        // Extract rows, potentially handling footer based on options
        let rowElements = Array.from(this.document.querySelectorAll('table tbody tr'));
        if ((_b = this.options) === null || _b === void 0 ? void 0 : _b.footer) {
            rowElements = rowElements.slice(0, -1); // Exclude the last row as footer
        }
        const rows = rowElements.map((row) => Array.from(row.querySelectorAll('td,th')).map((cell) => {
            const queryOnElements = 'input, textarea, button';
            const inputElements = cell.querySelectorAll(queryOnElements);
            if (inputElements.length > 0) {
                const element = inputElements[0];
                if (element instanceof this.dom.window.HTMLInputElement ||
                    element instanceof this.dom.window.HTMLTextAreaElement) {
                    return element.value;
                }
                if (element instanceof this.dom.window.HTMLButtonElement) {
                    return inputElements[0].getAttribute('aria-checked') || '';
                }
            }
            return cell.textContent ? cell.textContent.trim() : '';
        }));
        return this.buildData(rows, headers);
    }
    /**
     * Generates an array of header names from the table's thead section.
     * If a header element's text content is empty, it will be replaced with a unique identifier
     * in the format 'unknownX', where X is the number of missing headers encountered so far.
     *
     * @returns {string[]} - An array of header names, with empty headers replaced by 'unknownX'.
     */
    generateHeaders() {
        // Select both <th> and <td> elements within <thead>
        const headerElements = Array.from(this.document.querySelectorAll('table thead th, table thead td'));
        let unknownCount = 0;
        return headerElements.map((element) => {
            // Clean the text content of the header element
            const text = this.cleanHeaderText(element.textContent || '');
            if (text && text !== '') {
                return text;
            }
            else {
                return `Unknown${unknownCount++}`;
            }
        });
    }
    /**
     * Cleans header text by replacing newlines and multiple spaces with a single space.
     * @param text - The raw text extracted from the header element (th or td).
     * @returns A cleaned string with normalized spaces and no line breaks.
     */
    // function
    cleanHeaderText(text) {
        return text.replace(/\s+/g, ' ').trim();
    }
    buildData(rows, headers) {
        // Build the array of data
        const tableData = rows.map((row) => row.reduce((rowData, cell, index) => {
            rowData[headers[index]] = cell;
            return rowData;
        }, {}));
        return tableData;
    }
    buildWithFooter() {
        var _a;
        const tfoot = this.document.querySelector('table tfoot');
        if (!tfoot) {
            throw new Error('No <tfoot> element found in the table, but footer option is enabled.');
        }
        // Get footer headers
        let footerHeaders = Array.from(tfoot.querySelectorAll('th')).map((th) => { var _a; return ((_a = th.textContent) === null || _a === void 0 ? void 0 : _a.trim()) || ''; });
        // If footerHeaders is empty, fall back to options.headers
        if (footerHeaders.length === 0 && ((_a = this.options) === null || _a === void 0 ? void 0 : _a.header)) {
            footerHeaders = [...this.options.header]; // Use options.headers as fallback
        }
        // Get footer cells
        const footerCells = Array.from(tfoot.querySelectorAll('tr')).map((row) => Array.from(row.querySelectorAll('td')).map((cell) => {
            var _a;
            const queryOnElements = 'input, textarea, button';
            const inputElements = cell.querySelectorAll(queryOnElements);
            if (inputElements.length > 0) {
                const element = inputElements[0];
                if (element instanceof this.dom.window.HTMLInputElement ||
                    element instanceof this.dom.window.HTMLTextAreaElement) {
                    return element.value; // Extract value from input/textarea
                }
                if (element instanceof this.dom.window.HTMLButtonElement) {
                    const ariaChecked = element.getAttribute('aria-checked');
                    return ariaChecked || ((_a = element.textContent) === null || _a === void 0 ? void 0 : _a.trim()) || ''; // Return aria-checked or button text
                }
            }
            return cell.textContent ? cell.textContent.trim() : ''; // Return text content
        }));
        return this.buildData(footerCells, footerHeaders);
    }
}
exports.DataFrame = DataFrame;
